#!/usr/bin/python

from arsenal_lib import *

if len(sys.argv) < 2:
    sys.stderr.write("Usage: " + sys.argv[0] + " <source-package> [source-package ...]\n")
    exit(1)

opt_dryrun  = 0
max_age     = 31
total_count = 0
source_pkgs = sys.argv[1:]
new_status  = "Invalid"
arsenal     = Arsenal()
d           = arsenal.load_project("ubuntu")

message = """
We're closing this bug since it is has been some time with no response from the original reporter.  However, if the issue still exists please feel free to reopen with the requested information.  Also, if you could, please test against the latest development version of Ubuntu, since this confirms the bug is one we may be able to pass upstream for help.
"""

regex_desc_xorglog = re.compile('X.Org X Server')

total = 0
for source_pkg in source_pkgs:
    count = 0
    
    for bugtask in d.searchTasks(search_text=source_pkg, status="Incomplete (without response)"):
        if bugtask.bug_target_display_name != source_pkg + " (Ubuntu)":
            continue

        tries = 5
        while tries > 0:
            tries -= 1
            try:
                bug = ArsenalBug(bugtask.bug, arsenal.launchpad)

                # Skip if not old enough
                if bug.age_last_message() < max_age:
                    tries = 0
                    continue

                # Skip if bug is a Wishlist
                if bugtask.importance == "Wishlist":
                    print "Skipping wishlist bug",bug.id
                    tries = 0
                    continue

                # Skip if bug is assigned to someone
                if bugtask.assignee:
                    #print bug.id,"skipped since assigned to",bugtask.assignee.name
                    tries = 0
                    continue

                # Skip if bug already has an upstream tasks, in which case need to review/expire manually
                if bugtask.bug.bug_watches:
                    #print bug.id,"skipped since has upstream bug watches"
                    tries = 0
                    continue

                print bug.id, " ", bugtask.status, " ", bug.title, " [Old bug: ", bug.age_last_message(), " days]"

                bug.append_comment(message)
                bugtask.transitionToStatus(status = new_status)
                count = count + 1
                total = total + 1
                print " ---> " + new_status

            except HTTPError, e:
                print "*** Encountered HTTP error processing ", source_pkg, " ***"
                print e.content
                time.sleep(4)
                arsenal.reset()
                d = arsenal.load_project("ubuntu")


    print count, " bugs changed for ", source_pkg

print total, " total bugs changed"
