#!/usr/bin/python

# This script is meant to be periodically run against all of the X.org
# bugs to modify titles to add details, fix problems, and clarify.
# This is intended to help simplify forwarding the bugs upstream.

from arsenal_lib import *

if len(sys.argv) < 2:
    sys.stderr.write("Usage: " + sys.argv[0] + " <source-package>\n")
    exit(1)

source_pkgs = sys.argv[1:]
arsenal     = Arsenal()
d           = arsenal.load_project("ubuntu")

def cleanup_intrepid(title):
    # It's unnecessary to have [intrepid] in the title
    re_intrepid = re.compile("\[intrepid\] *", re.IGNORECASE)
    if re_intrepid.search(title):
        return re_intrepid.sub("", title)
    return title

def cleanup_intel(title):
    re_number = re.compile("^\[\d\d")
    if re_number.search(title):
        return "[i" + title[1:]

    re_bracket = re.compile("^\[[\w\,]*\d")
    if re_bracket.search(title):
        # Appears to already have been tagged
        return title

    re_NN5 = re.compile("([89]\d5)")
    m = re_NN5.search(title)
    if m:
        return "[i" + m.group(1) + "] " + title

    re_830 = re.compile("82830")
    if re_830.search(title):
        return "[i830] " + title

    re_g45 = re.compile("(gm*45|4500MHD)", re.IGNORECASE)
    if re_g45.search(title):
        return "[g45] " + title

    re_g35 = re.compile("gm*35", re.IGNORECASE)
    if re_g35.search(title):
        return "[g35] " + title

    return title

for pkg in source_pkgs:
    for bugtask in d.searchTasks(search_text=pkg):
        if bugtask.bug_target_display_name != pkg + " (Ubuntu)":
            continue

        bug = ArsenalBug(bugtask.bug, arsenal.launchpad)
        print "%-8d  %-12s  %s" % (bug.id, bugtask.status, bug.title)

        title = cleanup_intrepid(bug.title)

        if pkg == "xserver-xorg-video-intel":
            title = cleanup_intel(title)

        if title and title != bug.title:
            bug.bug.title = title
            bug.bug.lp_save()
            print "  ---------> Renamed to %s" % (title)

# TODO:
#   * s/du[ae]l *(head|display)/dual monitor/
#   * s/duel monitor/dual monitor/
#   * s/kayboard/keyboard/
#   * s/start up/startup/
#   * s/ X / Xorg /
#   * s/ res / resolution /
#   * s/ (hang|lock *up) / freeze /
#   * s/ (hangs|locks *up) / freezes /
#   * s/ eating / using /
#   * s/ upon / on /
#   * s/ garbled / corrupted /
#   * s/ sluggish / slow /
#   * s/ sluggishness / slowness /
#   * s/artefact/artifact/
#   * s/doesn\'t/does not/
#   * s/malfunctions/does not work/
#   * s/nonfunctional/does not work/
#   * s/(weird|wrong|craz+y|awful|borked|messed-up|unusable|broken)/incorrect/
#   * s/unreadable/incorrectly displayed/
#   * s/does not startup/fails to start/
#   * s/does not load at startup/fails to start/
#   * s/freezes after logging in/fails to login/
#   * s/screen illumination/backlight/
#   * s/gdm restart/Xorg restart/
#   * s/segmentation fault/SIGSEGV/
#   * s/segfault/SIGSEGV/
#   * s/out of scan range/out of range/
