#!/usr/bin/python

from arsenal_lib import *

source_pkg  = "xserver-xorg-video-intel"
arsenal     = Arsenal()
d           = arsenal.load_project("ubuntu")
new_status  = "Incomplete"

message = """
[This is an automatic notice.]

We'd like to forward your bug upstream, however upstream requires
that you first test it against their newer driver code.

To save you the effort of building the driver from source, we've built
packages for the driver and its new dependencies.

So you have a couple options:

 1.  Download and test .debs for intrepid, from:
     https://edge.launchpad.net/~intel-gfx-testing/+archive

 -or-

 2.  Download and test the Jaunty alpha-2 (or newer) Live CD,
     (which includes a beta of the new xserver 1.6 as well).
     See http://cdimage.ubuntu.com/releases/9.04/ for ISOs


Thanks ahead of time!  You can simply reply to this email to report your
findings.


P.S., if you wish to forward your bug upstream yourself, please follow
these directions to do so:
  http://intellinuxgraphics.org/how_to_report_bug.html
"""

re_8xx = re.compile("\[i8", re.IGNORECASE)
re_hardy = re.compile("hardy", re.IGNORECASE)
re_jaunty = re.compile("jaunty", re.IGNORECASE)
re_quirk = re.compile("quirk", re.IGNORECASE)
re_workflow = re.compile("^Please ", re.IGNORECASE)

count = 0
for bugtask in d.searchTasks(search_text=source_pkg, status="Incomplete (with response)"):
    if bugtask.bug_target_display_name != source_pkg + " (Ubuntu)":
        continue

    bug = ArsenalBug(bugtask.bug, arsenal.launchpad)

    if bugtask.status == "In Progress" or bugtask.status == "Fix Committed":
        print "%-8s %-10s %-12s %s" % (bug.id, "[INPROGRESS]", bugtask.status, bug.title)
        continue

    if bugtask.importance == "Wishlist":
        print "%-8s %-10s %-12s %s" % (bug.id, "[WISHLIST]", bugtask.status, bug.title)
        continue

    if re_8xx.search(bug.title):
        print "%-8s %-10s %-12s %s" % (bug.id, "[8xx CHIP]", bugtask.status, bug.title)
        continue

    if re_hardy.search(bug.title):
        print "%-8s %-10s %-12s %s" % (bug.id, "[HARDY]", bugtask.status, bug.title)
        continue

    if re_jaunty.search(bug.title):
        print "%-8s %-10s %-12s %s" % (bug.id, "[JAUNTY]", bugtask.status, bug.title)
        continue

    if re_quirk.search(bug.title):
        print "%-8s %-10s %-12s %s" % (bug.id, "[QUIRK]", bugtask.status, bug.title)
        continue

    if re_workflow.search(bug.title):
        print "%-8s %-10s %-12s %s" % (bug.id, "[WORKFLOW]", bugtask.status, bug.title)
        continue

    bug.append_comment(message)
    bugtask.transitionToStatus(status = "New")
    bugtask.transitionToStatus(status = "Incomplete")
    count = count + 1
    print "%-8s %-10s %-12s %s" % (bug.id, "-retest-", bugtask.status, bug.title)


print count, " bugs changed for ", source_pkg

