#!/usr/bin/python

# This script is for automatic processing of replies to bugs marked
# Incomplete, to filter out ones where requested info has been provided
# from ones that got incorrectly set to with-response via a
# non-original-reporter's reply.

from arsenal_lib import *

if len(sys.argv) < 2:
    sys.stderr.write("Usage: " + sys.argv[0] + " <source-package>")
    exit(1)

opt_dryrun  = 0
total_count = 0
source_pkgs = sys.argv[1:]
arsenal     = Arsenal()
d           = arsenal.load_project("ubuntu")

total_count = 0

try:
    for source_pkg in source_pkgs:
        count=0
        print "Processing ", source_pkg
        for bugtask in d.searchTasks(search_text=source_pkg, status="Incomplete (with response)"):
            if bugtask.bug_target_display_name != source_pkg + " (Ubuntu)":
                continue

            bug = ArsenalBug(bugtask.bug, arsenal.launchpad)

            print bug.id, "  ", bugtask.status, "  ", bug.title

            # If has tag needs-*, see if those are attached now
            # If so, remove tag.  If no more needs-* tags, set to Confirmed
            needs_xorglog = False
            has_xorglog = False
            if bug.has_tag('needs-xorglog'):
                needs_xorglog = True
                if bug.has_attachment('Xorg.0.log'):
                    bug.remove_tag('needs-xorglog')
                    has_xorglog = True
            else:
                has_xorglog = True

            needs_lspci_vvnn = False
            has_lspci_vvnn = False
            if bug.has_tag('needs-lspci-vvnn'):
                needs_lspci_vvnn = True
                if bug.has_attachment('lspci-vvnn.txt'):
                    bug.remove_tag('needs-lspci-vvnn')
                    has_lspci_vvnn = True
            else:
                has_xorglog = True

            # If all conditions met, set to Confirmed
            if (needs_xorglog or needs_lspci_vvnn) and \
               (has_xorglog and has_lspci_vvnn):
                # TODO:  Test!
                # bugtask.transitionToStatus(status = "Confirmed")
                print " --> Confirmed"

            # TODO:  Review comments since bug was set to Incomplete
            #        Note the original reporter
            #        If no one replying to the bug was the original reporter
            #        If all non-original-reporter repliers are bug triagers, reset to Incomplete
            #        Has the original reporter said "fixed"/"works now"/"can be closed"?
            #        If has lots of commenters, add caution that many people may simply
            #          be seeing the same symptoms, not the same bug.  Tag it.

            # Go through attachments
            # TODO:  Some of this attachment handling logic should be done in arsenal_lib
            #        Probably needs a new ArsenalAttachment wrapper class
            # TODO:  It would be nice to define pre-conditions in a 'rules' file, that
            #        arsenal could then apply to do triaging more programmatically
            for a in bug.attachments:
                hosted_file = a.data
                # The attachment content is downloaded in full during the call to
                # open(), so this can be slow.
                hosted_file_buffer = hosted_file.open()
                t = hosted_file_buffer.filename
                #content = hosted_file_buffer.read()

            # Is a patch attached?  If so, set to Confirmed directly

            # TODO:  Update bug
            if 1 == 2:
                count += 1
                total_count += 1

                new_status = "Confirmed"
                try:
                    bugtask.transitionToStatus(status = new_status)
                    print " ---> " + new_status
                except:
                    print " ERROR:  Failed to update bug ", bug.id
                    print "New status was:  ", new_status
                    raise

        print count, " bugs updated for ", source_pkg

except:
    print total_count, " bugs updated"
    raise

print total_count, " bugs updated"
