#!/usr/bin/python

# The purpose of this script is to conduct the initial review of
# incoming bugs and attempt to get them into a better shape for later
# scripts to handle.
#
# We want to categorize the bugs as best we can, and request appropriate
# information so later scripts can conduct their analysis.  We also want
# to kick out bug reports that aren't valid for handling by the X team.

from arsenal_lib import *

# Currently this script is suffering bug #336866 when trying to
# apply tags

if len(sys.argv) < 2:
    sys.stderr.write("Usage: " + sys.argv[0] + " <source-package>")
    exit(1)

opt_dryrun  = 0
total_count = 0
source_pkgs = sys.argv[1:]
arsenal     = Arsenal()
d           = arsenal.load_project("ubuntu")
new_status  = "Incomplete"

#arsenal.Debug(1)

automated = """
[This is an automated message.  Apologies if it has reached you inappropriately; please just reply to this message indicating so.]
"""

# Class to abstract some of the functions of crafting a good reply
# TODO:
#   * Reduce need for indicating automated message
#   * Make the message_* stuff be registered into this class somehow
#   * Vary the message used depending on quantity of items requested
#   * Control whether to specify it's an automated message based on length of reply
#   * Divide up phrasings more cleverly, esp. responses to stuff already included
#   * Include directions for if their request is a wishlist, packaging, or workflow bug
class ArsenalReply:
    def __init__(self, bug):
        self.bug = bug
        self.message = ""

    def set_message(self, message):
        self.message = message

    def append_message(self, message):
        self.message = self.message + message

    def get_message(self):
        return self.message

    def make_reply(self):
        if self.message != "":
            greeting = "Hi " + self.bug.owner_firstname + ",\n\n"
            return greeting + self.message + automated
        else:
            return ""

message_all = """
Please attach the output of `lspci -vvnn`, and attach your /var/log/Xorg.0.log (and maybe Xorg.0.log.old) file from after reproducing this issue.  If you've made any customizations to your /etc/X11/xorg.conf please attach that as well.
"""
message_img_only = """
Thanks for including an image to demonstrate the issue.  Could you also please attach the output of `lspci -vvnn`, and attach your /var/log/Xorg.0.log or Xorg.0.log.old file from after reproducing this issue. If you've made any customizations to your /etc/X11/xorg.conf please attach that as well.
"""
message_some = """
Thanks for including the attached files.  Could you also include your /var/log/Xorg.0.log (or Xorg.0.log.old) from after reproducing the issue?
"""
message_backtrace = """
Could you please collect a full backtrace from when this crash occurs?  Directions on collecting full backtraces are at https://wiki.ubuntu.com/X/Backtracing
"""
message_also_xorglog = """
Also, could you please attach your /var/log/Xorg.0.log from after reproducing the issue?
"""
message_also_lspci_vvnn = """
Please attach the output of `lspci -vvnn` too.
"""
message_xkeyboard_config = """
Please attach the output of 'setxkbmap -print' and 'xkbcomp :0 -'.
"""
message_old_bug = """
Thank you for taking the time to report this bug and helping to make Ubuntu better. You reported this bug a while ago and there hasn't been any activity in it recently. We were wondering is this still an issue for you? Can you try with the latest development release of Ubuntu?  (ISO CD images are available from http://cdimage.ubuntu.com/releases/)

If it remains an issue, could you also attach a new /var/log/Xorg.0.log?
Thanks in advance.
"""

# TODO:  Turn this into a dictionary of regex's
regex_image            = re.compile('(png|jpg)$',          re.IGNORECASE)
regex_xorglog          = re.compile('xorg.*log',           re.IGNORECASE)
regex_gdmlog           = re.compile(':.*log',              re.IGNORECASE)
regex_conf             = re.compile('conf',                re.IGNORECASE)
regex_strace           = re.compile('strace',              re.IGNORECASE)
regex_backtrace        = re.compile('ktrace',              re.IGNORECASE)
regex_lspci            = re.compile('lspci',               re.IGNORECASE)
regex_deps             = re.compile('Dependencies.txt',    re.IGNORECASE)
regex_subj_crash       = re.compile('(crash|SIGSEGV)',     re.IGNORECASE)
regex_subj_freeze      = re.compile('(freeze|locks* up|hangs)',     re.IGNORECASE)
regex_subj_workflow    = re.compile('Please (sync|merge)', re.IGNORECASE)
regex_subj_corruption  = re.compile('corruption', re.IGNORECASE)

# TODO:  Add hal, usb, uname detection

regex_desc_crash    = re.compile('ProblemType: Crash')
regex_desc_xorglog  = re.compile('X.Org X Server')

try:
    for source_pkg in source_pkgs:
        count=0
        print "Processing ", source_pkg
        for bugtask in d.searchTasks(search_text=source_pkg, status="New"):
            if bugtask.bug_target_display_name != source_pkg + " (Ubuntu)":
                continue

            # Skip wishlist bugs
            if bugtask.importance == "Wishlist":
                continue

            # TODO:  If bug has more than 3 ubuntu bugtasks, skip it

            bug = ArsenalBug(bugtask.bug, arsenal.launchpad)
            reply = ArsenalReply(bug)

            # TODO:  These bugs cause launchpad to issue a 500 error
            if bug.id == 337926:
                continue

            is_crash = 0
            has_image = 0
            has_xorglog = 0
            has_gdmlog = 0
            has_conf = 0
            has_strace = 0
            has_backtrace = 0
            has_lspci = 0
            has_deps = 0
            has_unknown = 0
            bug_title = bug.title
            bug_description = bug.description

            # TODO:  Turn the search commands into loop on a regex dictionary
            # TODO:  Move message construction logic into ArsenalReply
            if regex_subj_workflow.search(bug.title):
                print bug.id, "  " + bugtask.status + "  " + bug.title + "  [Workflow]"
                bug.append_tag("workflow")
                continue

            # TODO:  Analyze title and description for common issues, then set tags,
            #        title, and description accordingly.  Request appropriate files.
            #    * 'random' in title or description
            #    * 'does not boot'
            #    * 'rotate'
            #    * 'black'/'blank'/'no signal'/'no X* display'
            #    * high cpu
            #    * high memory
            #    * font size/dpi
            #    * rotate issues
            #    * performance/slow
            #    * external monitor
            #    * hotkey issues, ' key '
            #    * not using .* driver
            #    * 'resolution'/'\d+x\d+'
            #    * 'failed to install'
            #    * 'frequency too high'
            #    * 'video tearing'
            # TODO:  Itemize a list of tags that this script should use to classify bugs
            #        into appropriate categories for further handling by other scripts.
            #        For instance, black/blank screen bugs, high cpu bugs, etc.

            # TODO:  Fix common misspellings in title and description
            #    * interpid -> intrepid
            #    * crach -> crash

            if regex_subj_crash.search(bug.title) or \
                    regex_desc_crash.search(bug.description):
                is_crash = 1
                bug.append_tag("crash")

            if regex_desc_xorglog.search(bug.description):
                has_xorglog = 1

            if len(bug.attachments) == 0:
                print bug.id, "  ", bugtask.status, "  ", bug.title, "  [No attachments]"
                if not has_xorglog:
                    reply.set_message(message_all)
                    bug.append_tag("needs-xorglog")
                    bug.append_tag("needs-lspci-vvnn")
            else:
                print bug.id, "  ", bugtask.status, "  ", bug.title

                # Analyze what sorts of files are attached
                for a in bug.attachments:
                    hosted_file = a.data
                    # The attachment content is downloaded in full during the call to
                    # open(), so this can be slow.
                    hosted_file_buffer = hosted_file.open()
                    t = hosted_file_buffer.filename
                    #content = hosted_file_buffer.read()

                    if regex_image.search(t):
                        has_image = 1
                        print "  " + t + " [IMAGE]"
                    elif regex_xorglog.search(t):
                        has_xorglog = 1
                        print "  " + t + " [XORGLOG]"
                    elif regex_gdmlog.search(t):
                        has_gdmlog = 1
                        print "  " + t + " [GDMLOG]"
                    elif regex_conf.search(t):
                        has_conf = 1
                        print "  " + t + " [CONF]"
                    elif regex_strace.search(t):
                        has_strace = 1
                        print "  " + t + " [STRACE]"
                    elif regex_backtrace.search(t):
                        has_backtrace = 1
                        print "  " + t + " [BACKTRACE]"
                    elif regex_lspci.search(t):
                        has_lspci = 1
                        print "  " + t + " [LSPCI]"
                    elif regex_deps.search(t):
                        has_deps = 1
                        print "  " + t + " [DEPENDENCIES]"
                    else:
                        print "  " + t
                        has_unknown = 1

                    # TODO:  Search bug comments and Xorg.0.log's for lspci info, and if not, ask for it

                if not has_xorglog and not has_unknown:
                    bug.append_tag("needs-xorglog")
                    if has_image:
                        reply.set_message(message_img_only)
                    else:
                        reply.set_message(message_some)

            # Is a patch attached?  If so, set to Confirmed directly; maybe assign me?

            if bug.age_last_message() > 120:
                print " + Old bug over ", bug.age_last_message(), " days"
                reply.set_message(message_old_bug)
                bug.append_tag("needs-verification")

#            if not has_lspci:
#                reply.append_message(message_also_lspci_vvnn)
#                bug.append_tag("needs-lspci-vvnn")

# TODO:  Differentiate between crashes with and without full backtraces
#            if is_crash:
#                reply.append_message(message_backtrace)

            # Try to identify packaging bugs
            # Subject =~ "depend on"|"wishlist"|"^Please"|"needs"

            if reply.get_message():
                count += 1
                total_count += 1

                if opt_dryrun:
                    continue

                text = ""
                try:
                    text = reply.make_reply()
                    bug.append_comment(text)
                    bugtask.transitionToStatus(status = new_status)
                    print " ---> " + new_status
                except:
                    print " ERROR:  Failed to update bug ", bug.id
                    print "Message was:  ", text
                    print "New status was:  ", new_status
                    raise

            elif has_backtrace and has_xorglog and has_lspci:
                count += 1
                total_count += 1
                bugtask.transitionToStatus(status = "Confirmed")
                print " --> Confirmed (has backtrace and log)"

            elif has_lspci and has_xorglog:
                count += 1
                total_count += 1
                bugtask.transitionToStatus(status = "Confirmed")
                print " --> Confirmed (has log and pci id info)"

        print count, " bugs updated for ", source_pkg

except:
    print total_count, " bugs updated"
    raise

print total_count, " bugs updated"
