#!/usr/bin/python

# The purpose of this script is to move bugs out of the
# xorg package into more appropriate ones

from arsenal_lib import *
import httplib2
from launchpadlib.errors import HTTPError

total_count = 0
source_pkg  = "xorg"
arsenal     = Arsenal()
d           = arsenal.load_project("ubuntu")

def transitionToTarget(bugtask, package_name):
    package = d.getSourcePackage(name=package_name)
    if package == None:
        print "Error:  Could not find source package ",package_name
        return None
    bugtask.transitionToTarget(target = package)
    return 1

regex_fglrx       = re.compile('fglrx', re.IGNORECASE)
regex_radeon1     = re.compile(' (radeon|ati)', re.IGNORECASE)
regex_radeon2     = re.compile('(radeon|ati) ', re.IGNORECASE)
regex_nvidia      = re.compile('(nvidia|GeForce)', re.IGNORECASE)
regex_intel       = re.compile('intel', re.IGNORECASE)
regex_crash       = re.compile('(crash|abort|seg.*fault|lock *up|hang|restart|freeze)', re.IGNORECASE)
regex_xserver     = re.compile('(xserver|x server)', re.IGNORECASE)
regex_blank1      = re.compile('(blank|black) screen', re.IGNORECASE)
regex_blank2      = re.compile('screen .*(blank|black)', re.IGNORECASE)
regex_resolution  = re.compile('(resolution|res )', re.IGNORECASE)
regex_highcpu     = re.compile('(\d\d%.*cpu|high cpu)', re.IGNORECASE)
regex_keyboard1   = re.compile(' (key|keys|keyboard|layout)', re.IGNORECASE)
regex_keyboard2   = re.compile('(key|keys|keyboard|layout) ', re.IGNORECASE)
regex_mouseinput  = re.compile('(mouse.*input|touchpad)', re.IGNORECASE)
regex_mouseevents = re.compile('mouse.*events', re.IGNORECASE)
regex_monitor1    = re.compile('detect.*monitor', re.IGNORECASE)
regex_opengl      = re.compile('(opengl| gl )', re.IGNORECASE)
regex_compiz      = re.compile('(compiz|kwin|desktop effects)', re.IGNORECASE)

# TODO "WARNING: gdm_slave_xioerror_handler" in description

print "=== " + source_pkg + " ==="
for bugtask in d.searchTasks(search_text=source_pkg, assignee=None, importance="Undecided"):
    if bugtask.bug_target_display_name != source_pkg + " (Ubuntu)":
        continue

    tries = 3
    while tries > 0:
        tries -= 1
        try:
            bug = ArsenalBug(bugtask.bug, arsenal.launchpad)
            text = "%-7s %-11s %s" % (bug.id, bugtask.status, bug.title)

            # Bugs with a significant status should stay
            if bugtask.status == "Triaged" or bugtask.status == "In Progress" or bugtask.status == "Fix Committed":
                tries = 0
                continue

            if regex_intel.search(bug.title):
                print "Intel       -->  ", text
                transitionToTarget(bugtask, "xserver-xorg-video-intel")
                total_count += 1
                tries = 0
                continue

            if regex_nvidia.search(bug.title):
                print "nvidia      -->  ", text
                transitionToTarget(bugtask, "nvidia-graphics-drivers-180")
                total_count += 1
                tries = 0
                continue

            if regex_fglrx.search(bug.title):
                print "fglrx       -->  ", text
                transitionToTarget(bugtask, "fglrx-installer")
                total_count += 1
                tries = 0
                continue

            if regex_radeon1.search(bug.title) or regex_radeon2.search(bug.title):
                print "-ati        -->  ", text
                transitionToTarget(bugtask, "xserver-xorg-video-ati")
                total_count += 1
                tries = 0
                continue

            if regex_keyboard1.search(bug.title) or regex_keyboard2.search(bug.title):
                print "keyboard    -->  ", text
                transitionToTarget(bugtask, "xserver-xorg-input-evdev")
                total_count += 1
                tries = 0
                continue

            if regex_mouseinput.search(bug.title) or regex_mouseevents.search(bug.title):
                print "evdev       -->  ", text
                transitionToTarget(bugtask, "xserver-xorg-input-evdev")
                total_count += 1
                tries = 0
                continue

            if regex_crash.search(bug.title):
                print "Crash!      -->  ", text
                transitionToTarget(bugtask, "xorg-server")
                total_count += 1
                tries = 0
                continue

            if regex_xserver.search(bug.title):
                print "xserver     -->  ", text
                transitionToTarget(bugtask, "xorg-server")
                total_count += 1
                tries = 0
                continue

            if regex_blank1.search(bug.title) or regex_blank2.search(bug.title):
                print "Blank!      -->  ", text
                # TODO:  Transition to appropriate driver
                #total_count += 1
                tries = 0
                continue

            if regex_highcpu.search(bug.title):
                print "High CPU!   -->  ", text
                #total_count += 1
                tries = 0
                continue

            if regex_resolution.search(bug.title):
                print "Resolution! -->  ", text
                # TODO:  Could be driver, or xserver...
                #total_count += 1
                tries = 0
                continue

            if regex_monitor1.search(bug.title):
                print "Monitor     -->  ", text
                #total_count += 1
                tries = 0
                continue

            if regex_opengl.search(bug.title):
                print "opengl      -->  ", text
                transitionToTarget(bugtask, "mesa")
                total_count += 1
                tries = 0
                continue

            if regex_compiz.search(bug.title):
                print "compiz      -->  ", text
                transitionToTarget(bugtask, "compiz")
                total_count += 1
                tries = 0
                continue

            print "Unknown          ", text
            tries = 0
            
        except HTTPError, e:
            print "*** Exception: ",e.content

print total_count, " total bugs re-packaged"

# 167
# 178

